@@ -1,5 +1,5 @@
 /*
- * $Id: alarm.c,v 1.7 2004/03/16 23:04:04 adamf Exp $
+ * $Id: alarm.c,v 1.9 2005/04/09 00:20:12 adamf Exp $
  *
  * Copyright (C) Adam Feakin <adamf@snika.uklinux.net>
  *
@@ -40,6 +40,7 @@
 
 #include <string.h>
 #include <stdio.h>
+#include <errno.h>
 #include <xmms/plugin.h>
 #include <xmms/xmmsctrl.h>
 #include <xmms/configfile.h>
@@ -723,16 +724,29 @@ GeneralPlugin *get_gplugin_info()
  * a thread safe sleeping function -
  * and it even works in solaris (I think)
  */
-static void threadsleep(float x)
+static void threadsleep(float x, int id)
 {
    struct timespec rqtp, rmtp;
+   int nanosleep_return = 0;
 
-   DEBUG("threadsleep: waiting %f seconds\n", x);
+   DEBUG("[%d] threadsleep: waiting %f seconds\n", id, x);
 
    rqtp.tv_sec = x;
    rqtp.tv_nsec = (int)((float)(x - (int)x) * 1000000000.0);
 
-   nanosleep(&rqtp, &rmtp);
+   nanosleep_return = nanosleep(&rqtp, &rmtp);
+   while(nanosleep_return)
+   {
+     if(errno == EINTR)
+     {
+        DEBUG("[%d] nanosleep returned %d, errno = %d...restarting\n", id, nanosleep_return, errno);
+        rqtp = rmtp;
+        nanosleep_return = nanosleep(&rqtp, &rmtp);
+     }
+     else
+       DEBUG("[%d] nanosleep returned something bad (I know, not a very helpful message)...\n", id);
+   }
+   DEBUG("[%d] finished sleeping\n", id);
 
    return;
 }
@@ -777,18 +791,22 @@ static void *alarm_fade(void *arg)
    diff = vols->end - vols->start;
    adiff = abs(diff);
 
+   DEBUG("Volume diff = %d\n", diff);
+
    /* Are we going up or down? */
    if(diff < 0)
      inc = -1;
    else
      inc = 1;
+
+   DEBUG("Volume inc = %d\n", inc);
    
    xmms_remote_set_main_volume(alarm_plugin.xmms_session, (gint)vols->start);
    //for(i=0;i<(vols->end - vols->start);i++)
    for(i=0;i<adiff;i++)
    {
      //threadsleep((gfloat)fading / (vols->end - vols->start));
-     threadsleep((gfloat)fading / (gfloat)adiff);
+     threadsleep((gfloat)fading / (gfloat)adiff, S_FADER);
      xmms_remote_set_main_volume(alarm_plugin.xmms_session, 
          (gint)(xmms_remote_get_main_volume(alarm_plugin.xmms_session) + inc));
    }
@@ -809,6 +827,7 @@ static void *alarm_stop_thread( void *args )
    gint currvol;
    fader fade_vols;
    pthread_t f_tid;
+   float sleeptime;
 
    DEBUG("alarm_stop_thread\n");
 
@@ -816,7 +835,9 @@ static void *alarm_stop_thread( void *args )
    /* sleep for however long we are meant to be sleeping for until
     * its time to shut up
     */
-   threadsleep(((stop_h * 60) + stop_m) * 60);
+   sleeptime = (((float)stop_h * 60.0) + (float)stop_m) * 60.0;
+   DEBUG("alarm_stop sleeptime is %.3fs\n", sleeptime);
+   threadsleep(sleeptime, S_STOP);
 
    DEBUG("alarm_stop triggered\n");
 
@@ -859,15 +880,12 @@ static void *alarm_start_thread(void *args)
    unsigned int play_start = 0;
    guint today;
 
-   /* give it time to set start_tid to something */
-   threadsleep(1);
-
    while(start_tid != 0)
    {
      /* sit around and wait for the faders to not be doing anything */
      DEBUG("Waiting for fader to be unlocked..");
      pthread_mutex_lock(&fader_lock);
-     DEBUG("Ok\n");
+     DEBUG("fader unlocked\n");
      pthread_mutex_unlock(&fader_lock);
 
      DEBUG("Getting time\n");
@@ -885,7 +903,7 @@ static void *alarm_start_thread(void *args)
 
      if(alarm_conf.day[today].flags & ALARM_OFF)
      {
-       threadsleep(8.5);
+       threadsleep(8.5, S_MAIN);
        continue;
      }
      else
@@ -909,10 +927,14 @@ static void *alarm_start_thread(void *args)
      DEBUG("Checking time (%d:%d)\n", currtime->tm_hour, currtime->tm_min);
      if((currtime->tm_hour != alarm_h) || (currtime->tm_min != alarm_m))
      {
-       threadsleep(8.5);
+       threadsleep(8.5, S_MAIN);
        continue;
      }
 
+     /* if we are here then it must bt alarm time ... */
+     DEBUG("alarm time\n");
+
+
      if(cmd_on == TRUE)
      {
        DEBUG("Executing %s, cmd_on is true\n", cmdstr);
@@ -1030,16 +1052,16 @@ static void *alarm_start_thread(void *args)
           while(time(NULL) < (play_start + 61))
           {
             DEBUG("Waiting until out of starting minute\n");
-            threadsleep(5.0);
+            threadsleep(5.0, S_MAIN);
           }
           DEBUG("OK\n");
       }
       /* loop until we are out of the starting minute */
       while(time(NULL) < (play_start + 61))
       {
-        threadsleep(5.0);
+        threadsleep(5.0, S_MAIN);
       }
-      threadsleep(fading);
+      threadsleep(fading, S_MAIN);
    }
 
    DEBUG("Main thread has gone...\n");
